function [kx, ky, k] = kvalues(nbX, nbY, dpx, dpy, FFTPaddingFactor)
% Find the wavenumber coordinates

% Input Parameters
%   nbX (scalar): dimension of grid in x direction
%   nbY (scalar): dimension of grid in y direction
%   dpx (in m): real space pixel spacing in x direction
%   dpy (in m): real space pixel spacing in y direction
%   FFTPaddingFactor (scalar): padding factor of FFT

% Output Parameters
%   kx (meshgrid): k vector in x direction
%   ky (meshgrid): k vector in y direction
%   k  (meshgrid): k vector

    nbKx = nbX*FFTPaddingFactor;
    nbKy = nbY*FFTPaddingFactor;
    dkx = 2*pi/(dpx*nbKx);
    dky = 2*pi/(dpy*nbKy);

    if mod(nbX,2) == 0 
        % even
        % DC component is not center pixel 
        kx_array = -dkx*(nbKx/2) : dkx : dkx*(nbKx/2-1); 
    else
        % odd
        % DC component is center pixel
        kx_array = -dkx*(nbKx/2-0.5) : dkx : dkx*(nbKx/2-0.5); 
    end
    if mod(nbY,2) == 0 
        % even
        % DC component is not center pixel   
        ky_array = -dky*(nbKy/2) : dky : dky*(nbKy/2-1); 
    else
        % odd
        % DC component is center pixel
        ky_array = -dky*(nbKy/2-0.5) : dky : dky*(nbKy/2-0.5); 
    end
    
    % A tiny 'epsilon' is added to
    % each freq. variable so as to avoid division by zero warning.
    eps = 1e-20;
    [kx,ky] = meshgrid(kx_array+eps,ky_array+eps);

    k = sqrt(kx.^2 + ky.^2);
end

